@@ -25,19 +25,19 @@
 #include <cupt/versionstring.hpp>
 
 #include <internal/versionparse.hpp>
+#include <internal/common.hpp>
 
 namespace cupt {
 namespace cache {
 
-Package::Package(const string* binaryArchitecture)
-	: _binary_architecture(binaryArchitecture)
+Package::Package()
 {}
 
 void Package::addEntry(const internal::VersionParseParameters& initParams)
 {
 	try
 	{
-		__merge_version(_parse_version(initParams));
+		__merge_version(*initParams.binaryArchitecturePtr, _parse_version(initParams));
 	}
 	catch (Exception& e)
 	{
@@ -61,15 +61,27 @@ vector< const Version* > Package::getVersions() const
 	return result;
 }
 
-static inline bool __is_installed(const Version* version)
+namespace {
+
+inline bool __is_installed(const Version* version)
 {
 	auto binaryVersion = dynamic_cast< const BinaryVersion* >(version);
 	return (binaryVersion && binaryVersion->isInstalled());
 }
 
-void Package::__merge_version(unique_ptr< Version >&& parsedVersion)
+void addVersionIdSuffix(string* dest, const string& suffix)
 {
-	if (!_is_architecture_appropriate(parsedVersion.get()))
+	*dest += internal::idSuffixDelimiter;
+	*dest += suffix;
+}
+
+const string installedSuffix = "installed";
+
+}
+
+void Package::__merge_version(const string& binaryArchitecture, unique_ptr< Version >&& parsedVersion)
+{
+	if (!_is_architecture_appropriate(binaryArchitecture, parsedVersion.get()))
 	{
 		return; // skip this version
 	}
@@ -81,8 +93,7 @@ void Package::__merge_version(unique_ptr< Version >&& parsedVersion)
 		{
 			// no way to know is this version the same as in repositories,
 			// until for example #667665 is implemented
-			parsedVersion->versionString += versionstring::idSuffixDelimiter;
-			parsedVersion->versionString += "installed";
+			addVersionIdSuffix(&parsedVersion->versionString, installedSuffix);
 			__parsed_versions.push_back(std::move(parsedVersion));
 		}
 		else
@@ -93,7 +104,7 @@ void Package::__merge_version(unique_ptr< Version >&& parsedVersion)
 			bool merged = false;
 			for (const auto& presentVersion: __parsed_versions)
 			{
-				if (!versionstring::sameOriginal(presentVersion->versionString, parsedVersionString))
+				if (!getOriginalVersionString(presentVersion->versionString).equal(parsedVersionString))
 				{
 					continue;
 				}
@@ -120,8 +131,7 @@ void Package::__merge_version(unique_ptr< Version >&& parsedVersion)
 				if (clashed)
 				{
 					static size_t idCounter = 0;
-					parsedVersion->versionString += versionstring::idSuffixDelimiter;
-					parsedVersion->versionString += format2("dhs%zu", idCounter++);
+					addVersionIdSuffix(&parsedVersion->versionString, format2("dhs%zu", idCounter++));
 				}
 				__parsed_versions.push_back(std::move(parsedVersion));
 			}
