@@ -41,13 +41,12 @@ struct StateData
 {
 	shared_ptr< const Config > config;
 	internal::CacheImpl* cacheImpl;
-	map< string, shared_ptr< const InstalledRecord > > installedInfo;
+	map<string, unique_ptr< const InstalledRecord >> installedInfo;
 
 	void parseDpkgStatus();
 };
 
-void parseStatusSubstrings(const string& packageName, const string& input,
-		const shared_ptr< InstalledRecord >& installedRecord)
+void parseStatusSubstrings(const string& packageName, const string& input, InstalledRecord* installedRecord)
 {
 	// status should be a triplet delimited by spaces (i.e. 2 ones)
 	internal::TagParser::StringRange current;
@@ -85,8 +84,6 @@ void parseStatusSubstrings(const string& packageName, const string& input,
 #define CHECK_FLAG(str, value) if (current.equal(BUFFER_AND_SIZE(str))) { installedRecord->flag = InstalledRecord::Flag:: value; } else
 		CHECK_FLAG("ok", Ok)
 		CHECK_FLAG("reinstreq", Reinstreq)
-		CHECK_FLAG("hold", Hold)
-		CHECK_FLAG("hold-reinstreq", HoldAndReinstreq)
 		{ // else
 			fatal2(__("malformed '%s' status indicator (for the package '%s')"), "error", packageName);
 		}
@@ -107,8 +104,6 @@ void parseStatusSubstrings(const string& packageName, const string& input,
 		CHECK_STATUS("unpacked", Unpacked)
 		CHECK_STATUS("half-configured", HalfConfigured)
 		CHECK_STATUS("half-installed", HalfInstalled)
-		CHECK_STATUS("post-inst-failed", PostInstFailed)
-		CHECK_STATUS("removal-failed", RemovalFailed)
 		CHECK_STATUS("triggers-pending", TriggersPending)
 		CHECK_STATUS("triggers-awaited", TriggersAwaited)
 		{ // else
@@ -208,8 +203,8 @@ void StateData::parseDpkgStatus()
 			{
 				fatal2(__("no package name in the record"));
 			}
-			auto installedRecord = std::make_shared< InstalledRecord >();
-			parseStatusSubstrings(packageName, status, installedRecord);
+			unique_ptr<InstalledRecord> installedRecord(new InstalledRecord());
+			parseStatusSubstrings(packageName, status, installedRecord.get());
 
 			if (packageHasFullEntryInfo(*installedRecord))
 			{
@@ -230,8 +225,7 @@ void StateData::parseDpkgStatus()
 			}
 
 			// add parsed info to installed_info
-			installedInfo.insert(pair< const string, shared_ptr< const InstalledRecord > >(
-					std::move(packageName), std::move(installedRecord)));
+			installedInfo.emplace(std::move(packageName), std::move(installedRecord));
 		}
 	}
 	catch (Exception&)
@@ -263,16 +257,16 @@ State::~State()
 	delete __data;
 }
 
-shared_ptr< const State::InstalledRecord > State::getInstalledInfo(const string& packageName) const
+const State::InstalledRecord* State::getInstalledInfo(const string& packageName) const
 {
 	auto it = __data->installedInfo.find(packageName);
 	if (it == __data->installedInfo.end())
 	{
-		return shared_ptr< const InstalledRecord >();
+		return nullptr;
 	}
 	else
 	{
-		return it->second;
+		return it->second.get();
 	}
 }
 
@@ -302,7 +296,6 @@ vector< string > State::getReinstallRequiredPackageNames() const
 		const InstalledRecord::Flag::Type& flag = it->second->flag;
 		const InstalledRecord::Status::Type& status = it->second->status;
 		if (flag == InstalledRecord::Flag::Reinstreq ||
-				flag == InstalledRecord::Flag::HoldAndReinstreq ||
 				status == InstalledRecord::Status::HalfInstalled)
 		{
 			result.push_back(it->first);
@@ -319,9 +312,12 @@ string State::getArchitecture() const
 
 const string State::InstalledRecord::Status::strings[] = {
 	N__("not installed"), N__("unpacked"), N__("half-configured"), N__("half-installed"),
-	N__("config files"), N__("postinst failed"), N__("removal failed"), N__("installed")
+	N__("config files"), N__("installed"), N__("triggers-pending"), N__("triggers-awaited")
 };
 
+typedef State::InstalledRecord::Status SIS;
+static_assert(sizeof(SIS::strings)/sizeof(SIS::strings[0]) == SIS::Count, "status string misdefined");
+
 }
 }
 
